/*   This software was written by Gregory L Montgomery
 and is completely covered by the 
 GPL General Public License

 Date of release 1/7/1998 */

/* Programmer Gregory L Montgomery */
/*                                 */
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/malloc.h>
#include <linux/fcntl.h>

#include <asm/system.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/pgtable.h>

#define MAX_PORTNO 16

static unsigned char IO_Port_Buffer[MAX_PORTNO];

static long IoPort_write(struct inode * inode, struct file * file,
                         const char * buf, unsigned long count ){

        const char * bufptr = buf;
        int i;
	unsigned int minor = MINOR(inode->i_rdev);

	if(minor < MAX_PORTNO){
		/* IO_Port_Buffer[minor] = (get_fs_byte(bufptr+count-1)); */
                for(i=0; i < count; i++){
                      get_user(IO_Port_Buffer[minor],bufptr++);
	  	      outb_p(IO_Port_Buffer[minor], IOPORT_ADDRESS+minor);
                }
		return 1;
	}
	else {
	        printk(KERN_NOTICE "I/O Port: not port at  0x%X\n",IOPORT_ADDRESS+minor);
		return 0;
	}
} /* end of ioport_write() */

static long IoPort_read(struct inode * inode, struct file * file, 
                        char * buf, unsigned long  count){

        char * bufptr = buf;
        int i;
	unsigned int minor = MINOR(inode->i_rdev);

	if(minor < MAX_PORTNO){
                for(i = 0; i < count; i++){
		     IO_Port_Buffer[minor] = inb_p(IOPORT_ADDRESS+minor);
                     put_user(IO_Port_Buffer[minor],bufptr++);
		     /* put_fs_byte(IO_Port_Buffer[minor],bufptr+count-1); */
                }
		return 1;
	}
	else{
		printk(KERN_NOTICE "IoPort: no port at 0x%X\n",IOPORT_ADDRESS+minor);
		return 0;
	}
} /* end of ioport_read */

static struct file_operations ioport_fops = { 

  NULL,           /* IO_Port_lseek                   */
  IoPort_read,    /* IO_Port_read ; read an ioport   */
  IoPort_write,   /* IO_Port_write ; write an ioport */
  NULL,           /* IO_Port_readdir                 */
  NULL,           /* IO_Port_select                  */
  NULL,           /* IO_Port_ioctl                   */
  NULL,           /* IO_Port_mmap                    */
  NULL,           /* IO_Port_open                    */
  NULL,           /* IO_Port_release                 */


};

long IOPort_init(long kmem_start){

  int count;
  int ioport_initmode = 0xFF;

  if(register_chrdev(IOPORT_MAJOR, "IO Ports", &ioport_fops)){
    printk("unable to get major %d for IO-Port\n", IOPORT_MAJOR);
    return kmem_start;
  }
  else {
    printk("IO-Port driver installed first port at 0x%X\n", IOPORT_ADDRESS);
    /* Initialize all of the I/O Ports to an initial value of 255 */
    outb_p(0, IOPORT_ADDRESS); /* Turn off all the stuff */
    for (count = 1; count <= MAX_PORTNO; count++){
      outb_p(0xFF, IOPORT_ADDRESS+count);
      IO_Port_Buffer[count] = ioport_initmode;
    }
    return kmem_start;
  }
}
